from libs.enums.ImportType import ImportType, ImportTypeTranslation
from PyQt5.QtWidgets import QMainWindow, QFileDialog, QMessageBox
from libs.ui.PieChartWindow import PieChartWindow
from libs.import_fles import ImportFiles
from PyQt5.uic import loadUi
from libs.db import DB


class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        loadUi('libs/ui/ui_files/main_window.ui', self)
        self._db = DB()

        self.browseFileButton.clicked.connect(self._browse_file)
        self.importButton.clicked.connect(self._import_file)

        self.getSumAction.triggered.connect(self._get_total_cost_plan)
        self.exportPieAction.triggered.connect(self._show_pie)

        for import_type in ImportType:
            self.importType.addItem(ImportTypeTranslation.data[import_type.name], import_type)

    def _browse_file(self):
        """
        Выбрать файл
        """
        file_path = QFileDialog.getOpenFileName(self, 'Выберите файл Excel', '', 'Excel файлы (*.xlsx)')
        if file_path[0]:
            self.filePath.setText(file_path[0])

    def _show_pie(self):
        """
        Отобразить pie диаграмму
        """
        self._db.cursor.callproc('GetCostItemDetails')
        data = self._db.cursor.fetchall()
        self.pieChartWindow = PieChartWindow(data)
        self.pieChartWindow.show()

    def _get_total_cost_plan(self):
        """
        Вывести на экран суммарная себестоимость выпуска изделий на план
        """
        self._db.cursor.callproc('GetTotalCostOfPlan')
        data = self._db.cursor.fetchone()
        total_cost = "{:,.2f}".format(float(data[0])).replace(",", " ")
        QMessageBox.information(self, "Результат", f"Суммарная себестоимость выпуска изделий на план: {total_cost}", QMessageBox.Ok)

    def _import_file(self):
        """
        Импортировать файл
        """
        if not self.filePath.text():
            QMessageBox.warning(self, "Внимание", "Файл не выбран.", QMessageBox.Ok)
            return

        importer = ImportFiles()
        importer.set_settings(self.filePath.text(), self.importType.itemData(self.importType.currentIndex()))
        if importer.import_xls():
            QMessageBox.information(self, "Успех!", "Импорт завершён.", QMessageBox.Ok)
        else:
            QMessageBox.error(self, "Ошибка", "Что-то пошло не так.", QMessageBox.Ok)



from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from PyQt5.QtWidgets import QMainWindow, QVBoxLayout, QWidget, QPushButton
from PyQt5.QtWidgets import QFileDialog
import matplotlib.pyplot as plt


class PieChartWindow(QMainWindow):
    def __init__(self, data):
        super().__init__()
        self.setWindowTitle("Доля каждой статьи затрат в общей себестоимости выпуска")

        self.figure = plt.figure()
        self.canvas = FigureCanvas(self.figure)

        self.layout = QVBoxLayout()
        widget = QWidget()
        widget.setLayout(self.layout)
        self.setCentralWidget(widget)

        self.layout.addWidget(self.canvas)
        self.plot(data)

        self.saveButton = QPushButton("Сохранить изображение")
        self.saveButton.clicked.connect(self.save_plot)
        self.layout.addWidget(self.saveButton)

    def plot(self, data):
        labels = [x[0] for x in data]
        sizes = [x[1] for x in data]

        ax = self.figure.add_subplot(111)
        ax.pie(sizes, labels=labels, autopct='%1.1f%%', startangle=140)
        ax.axis('equal')
        self.canvas.draw()

    def save_plot(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        fileName, _ = QFileDialog.getSaveFileName(self, "Сохранить изображение", "", "PNG Files (*.png);;All Files (*)",
                                                  options=options)
        if fileName:
            self.figure.savefig(fileName)



from libs.enums.ImportType import ImportType
from PyQt5.QtWidgets import QMessageBox
from libs.db import DB
import openpyxl


class ImportFiles:
    def __init__(self):
        self._file_path = None
        self._import_type = None
        self._using_ids = None
        self._db = DB()

    def _error(self, error):
        """
        Выводит ошибку в MessageBox
        :param error: Текст ошибки
        """
        QMessageBox.error(self, "Ошибка", f"{error}", QMessageBox.Ok)

    def import_xls(self):
        """
        Импортироват xls файл в базу данных
        :return: Успешность импорта файла в базу данных
        """
        try:
            if self._file_path is not None:
                sheet = openpyxl.load_workbook(self._file_path).active

                if self._import_type == ImportType.COST_ITEMS:
                    for row in sheet.iter_rows(values_only=True):
                        self._import_to_db(row[0], 'cost_items')
                elif self._import_type == ImportType.PRODUCTS:
                    for row in sheet.iter_rows(values_only=True):
                        self._import_to_db(row[0], 'products')
                elif self._import_type == ImportType.CALCULATION:
                    self._process_calculation_data(sheet)
                elif self._import_type == ImportType.PRODUCTION_PLAN:
                    self._process_production_plan_data(sheet)
                else:
                    return False

            return True
        except Exception as e:
            self._error(e)

    def _process_production_plan_data(self, sheet):
        """
        Обработать данные планы выпуска
        :param sheet: Строчка таблицы
        """
        try:
            for row in sheet.iter_rows(values_only=True):
                if self._using_ids:
                    product_id, quantity = row
                else:
                    product_name, quantity = row
                    product_id = self._db.get_id_by_name(product_name, 'products')[0]

                self._db.cursor.execute(
                    f"INSERT INTO `production_plan`(`product_id`, `quantity`) VALUES ('{product_id}','{quantity}');")
                self._db.connection.commit()
        except Exception as e:
            self._error(e)

    def _process_calculation_data(self, sheet):
        """
        Обработать данные калькуляции
        :param sheet: Строчка таблицы
        """
        try:
            for row in sheet.iter_rows(values_only=True):
                if self._using_ids:
                    product_id, cost_item_id, amount = row
                else:
                    product_name, cost_item_name, amount = row
                    product_id = self._db.get_id_by_name(product_name, 'products')[0]
                    cost_item_id = self._db.get_id_by_name(cost_item_name, 'cost_items')[0]

                self._db.cursor.execute(
                    f"INSERT INTO `calculation`(`product_id`, `cost_item_id`, `amount`) VALUES ('{product_id}','{cost_item_id}','{amount}');")
                self._db.connection.commit()
        except Exception as e:
            self._error(e)

    def set_settings(self, file_path: str, import_type: ImportType = ImportType.COST_ITEMS, using_ids: bool = False):
        """
        Установить настройки
        :param file_path: Путь до файла
        :param import_type: Тип импорта (по умолчанию ImportType.COST_ITEMS)
        :param using_ids: Есть ли в файле id (по умолчанию False)
        """
        self._file_path = file_path
        self._import_type = import_type
        self._using_ids = using_ids

    def _import_to_db(self, name, table):
        """
        Импортировать name в таблицу
        :param name: Значение name
        :param table: Имя таблиыч в базе данных
        :return:
        """
        self._db.cursor.execute(f"INSERT INTO `{table}`(`name`) VALUES ('{name}');")
        self._db.connection.commit()



from enum import Enum


class ImportType(Enum):
    COST_ITEMS = 0
    PRODUCTS = 1
    CALCULATION = 2
    PRODUCTION_PLAN = 3


class ImportTypeTranslation:
    data = {
        "COST_ITEMS": "Статьи затрат",
        "PRODUCTS": "Изделия",
        "CALCULATION": "Калькуляции",
        "PRODUCTION_PLAN": "Планы выпуска"
    }



import pymysql


class DB:
    def __init__(self):
        self.cursor = None
        self.connection = None
        self.connect()

    def connect(self):
        """
        Подключиться к базе данных
        :return:
        """
        self.connection = pymysql.connect(host="127.0.0.1", user="root", passwd="root", db="calculatio", charset="utf8")
        self.cursor = self.connection.cursor()

    def get_id_by_name(self, name, table):
        """
        Получить id по имени из таблицы
        :param name: Значение name
        :param table: Имя таблиыч в базе данных
        :return: Результат поиска
        """
        self.cursor.execute(f"SELECT * FROM `{table}` WHERE `name` = '{name}';")
        return self.cursor.fetchone()

    def __del__(self):
        self.cursor.close()
        self.connection.close()



from libs.ui.MainWindow import MainWindow
from PyQt5.QtWidgets import QApplication
import sys

if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec_())



from setuptools import setup, find_packages

setup(
    name='pymysqls',
    version='0.1',
    packages=find_packages(),
    install_requires=[
        'PyMySQL',
    ],
    package_data={
        '': ['pymysqls/1/*', 'pymysqls/2/*', 'pymysqls/1/lib/*', 'pymysqls/2/libs/*', 'pymysqls/2/libs/ui/*', 'pymysqls/2/libs/enums/*', 'pymysqls/2/libs/ui/ui_files/*'],
    },
    include_package_data=True,
)



from lib.db import DB
import sys
from PyQt6 import QtCore, QtGui, QtWidgets
from PyQt6.QtGui import QPixmap
from Ui_Form import Ui_Form


class MainWindow(QtWidgets.QWidget, Ui_Form):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.db = DB()
        self.render_data()

    def get_data(self):
        if self.db.con:
            try:
                cursor = self.db.con.cursor()
                query = f"SELECT service.name, service.price, data, status, service.img FROM orders LEFT JOIN service ON orders.id_service = service.id;"
                cursor.execute(query)
                data = cursor.fetchall()
                return data
            except Exception as e:
                print("Ошибка при выполнении запроса:", e)
                return []
        else:
            return []

    def render_data(self):
        data = self.get_data()
        for row in data:
            self.add_object(row)

    def add_object(self, data):
        obj = QtWidgets.QHBoxLayout()
        image = QtWidgets.QLabel()
        image.setMinimumSize(QtCore.QSize(150, 150))
        image.setMaximumSize(QtCore.QSize(150, 150))
        if data[4] != "":
            pixmap = QPixmap()
            pixmap.loadFromData(data[4])
            image.setPixmap(pixmap)
        image.setScaledContents(True)
        obj.addWidget(image)
        label = QtWidgets.QLabel()
        label.setText(f"{data[0]}\n{data[1]} руб.\n{data[2].strftime('%H:%M:%S %d.%m.%y')}\nСтатус: {data[3]}")
        obj.addWidget(label)
        self.verticalLayout.addLayout(obj)


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    window = MainWindow()
    window.show()
    sys.exit(app.exec())



# Form implementation generated from reading ui file 'Ui_Form.ui'
#
# Created by: PyQt6 UI code generator 6.6.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(526, 342)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(parent=Form)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 506, 322))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 1, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Приложение обслуживания"))



import mysql.connector

host = '46.188.59.18'
user = 'service_yvu'
password = 'd1Ha6rmu5Q/eDOwZ'
database = 'service_yvu'


class DB:
    def __init__(self):
        self.con = self.connect_to_mysql()

    def __del__(self):
        if self.con is not None:
            print("Отключение от БД...")
            self.con.close()

    def connect_to_mysql(self):
        try:
            connection = mysql.connector.connect(
                host=host,
                user=user,
                password=password,
                database=database
            )
            print("Подключение к БД...")
            return connection
        except Exception as e:
            print("Ошибка при подключении.", e)
            return None
